using Jypeli;
using Jypeli.Assets;

namespace AngryLego;

/// @author vesal
/// @version 16.10.2023
/// <summary>
/// 
/// </summary>
public class AngryLego : PhysicsGame
{
    
    private static readonly string[] lines = {
        "                        ",
        "                        ",
        "                        ",
        "                        ",
        "/                       ",
        "                        ",
        "                        ",
        "                        ",
        "                        ",
        "                        ",
        "                        ",
        "                        ",
        "                        ",
        "                        ",
        "                        ",
        "                        ",
        "                        ",
        "        =======         ",
        "        Y  Y  Y         ",
        "        X* X  X         ",
        "   *    X  X  X     *   ",
        "        X  X  X         ",
        "        X *X *X         ",
    };
    
    private static readonly int tileWidth = 800 / lines[0].Length;
    private static readonly int tileHeight = 480 / lines.Length;

    private PhysicsObject maila; // LuoMaila, KaannaMaila
    
    public override void Begin()
    {
        ClearGameObjects();
        ClearControls();
        Gravity = new Vector(0, -500);
        
        Level.Background.CreateGradient(Color.Blue, Color.White);
        
        TileMap tiles = TileMap.FromStringArray(lines);
        tiles.SetTileMethod('Y', LuoYlaSeina, Color.Wheat); // Aliohjelmien pitää itse lisätä, enemmän param
        tiles.SetTileMethod('X', LuoSeina, Color.Wheat); // Aliohjelmien pitää itse lisätä, enemmän param
        tiles.SetTileMethod('=', LuoKatto, Color.Red);
        tiles.SetTileMethod('/', LuoMaila, Color.Black);
        tiles.SetTileMethod('*', LuoVihollinen, Color.Pink);

        tiles.Execute(tileWidth, tileHeight);

        Level.CreateBorders();
        Camera.ZoomToLevel();

        Keyboard.Listen(Key.F1, ButtonState.Pressed, ShowControlHelp, "Show help");
        Keyboard.Listen(Key.Up, ButtonState.Pressed, KaannaMailaa, "Move up", 5.0);
        Keyboard.Listen(Key.Down, ButtonState.Pressed, KaannaMailaa, "Move down", -5.0);
        Keyboard.Listen(Key.Space, ButtonState.Pressed, PudotaPallo, "Drop ball");
        
        PhoneBackButton.Listen(ConfirmExit, "Lopeta peli");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
    }
    
    
    private void PudotaPallo()
    {
        PhysicsObject pallo = new PhysicsObject(tileWidth, tileWidth, Shape.Circle);
        pallo.Position = maila.Position + new Vector(tileWidth, maila.Height + tileWidth);
        pallo.Image = LoadImage("Igor");
        Add(pallo);
        AddCollisionHandler(pallo, "vihu", PalloOsui);
    }

    
    /// <summary>
    /// Apualiohjlema vihollisen räjäyttämiseksi ja poistamiseksi
    /// </summary>
    /// <param name="vihu"></param>
    private void PossautaVihu(IPhysicsObject vihu)
    {
        Explosion rajahdys = new Explosion(vihu.Width * 10);
        rajahdys.Position = vihu.Position;
        rajahdys.UseShockWave = false;
        Add(rajahdys);
        Remove(vihu);
    }
    
    
    /// <summary>
    /// Tapahtuma kun pallo osuu viholliseen
    /// </summary>
    /// <param name="pallo">pallo joka osui</param>
    /// <param name="vihu">vihollinen johon osuttiin</param>
    private void PalloOsui(PhysicsObject pallo, PhysicsObject vihu)
    {
        PossautaVihu(vihu);
    }
    
    
   /// <summary>
    /// Luodaan seinäelementti
    /// </summary>
    private void LuoSeina(Vector paikka, double leveys, double korkeus, Color vari)
    {
        PhysicsObject seina = new PhysicsObject(leveys-1, korkeus-1);
        seina.Position = paikka;
        seina.Color = Color.Wheat;
        seina.Tag = "rakenne";
        seina.Image = LoadImage("tiili");
        Add(seina);
    }


    /// <summary>
    /// Käännetään mailaa
    /// </summary>
    /// <param name="kulma">millä kulmalla käännetään</param>
    private void KaannaMailaa(double kulma)
    {
       maila.Angle += Angle.FromDegrees(kulma);
    }

   
    /// <summary>
    /// Luodaan isompi yläseinäelementti
    /// </summary>
    private void LuoYlaSeina(Vector paikka, double leveys, double korkeus, Color vari)
    {
        PhysicsObject seina = new PhysicsObject(leveys*1.8, korkeus-1);
        seina.Position = paikka;
        
        seina.Color = Color.Wheat;
        seina.Tag = "rakenne";
        seina.Image = LoadImage("tiili");
        Add(seina);
    }


    /// <summary>
    /// Luodaan kattoelementti.  Luodaan hieman ylisuureksi, jolloin liimautuu
    /// naapuriin kiinni.
    /// </summary>
    private void LuoKatto(Vector paikka, double leveys, double korkeus, Color vari)
    {
        PhysicsObject katto = new PhysicsObject(leveys-1, korkeus-1);
        katto.Position = paikka;
        katto.Color = vari;
        katto.Tag = "rakenne";
        katto.Image = LoadImage("tiili");
        Add(katto);
    }


    /// <summary>
    /// Luodaan maila, jolla palloja lyödään
    /// </summary>
    private void LuoMaila(Vector paikka, double leveys, double korkeus, Color vari)
    {
        maila = PhysicsObject.CreateStaticObject(leveys * 6, korkeus);
        maila.Position = paikka;
        maila.Color = vari;
        maila.Image = LoadImage("maila3");
        Add(maila);
    }


    /// <summary>
    /// Luodaan vihollinen, joka hajoaa osuessaan rekenteeseen
    /// </summary>
    private void LuoVihollinen(Vector paikka, double leveys, double korkeus, Color vari)
    {
        PhysicsObject vihu = new PhysicsObject(leveys / 2, leveys / 2, Shape.Circle);
        vihu.Position = paikka;
        vihu.Color = vari;
        // vihu.Collided += new NewCollisionHandler(VihuunOsui);
        vihu.Tag = "vihu";
        vihu.Image = LoadImage("Baby");
        Add(vihu);
    }


}